// Extend this model to be able to specify a jsonRoot attribute to get around the
// fact that Rails includes one with each JSON render.
var RailsModel = Backbone.Model.extend({
  jsonRoot: "",
  parse: function(response) { return response[this.jsonRoot]; }
});

// Extend this collection to be able to specify a jsonRoot attribute to get around the
// fact that Rails includes one with each JSON render. 
var RailsCollection = Backbone.Collection.extend({
  jsonRoot: "",
  parse: function(r) {
    _jsr = this.jsonRoot;
    var mapped = _.map(r, function(item) { return item[_jsr]; });
    return mapped;
  }
});

// Setup the current user
var User = RailsModel.extend({ jsonRoot: "user" });

var CurrentUser = User.extend({ url: _u("/api/current.json") });
var SearchInviteUsers = RailsCollection.extend({ 
  jsonRoot: "user", 
  url: function() { 
    u = _u("/api/users/search.json?query="+this.query); 
    if(this.exceptIds) {
      u += "&except="+this.exceptIds.join(",");
    } else {
      u += "&no_conns=true";
    }
    return u;
  },

  model: User
});

// Connections
var Connection = RailsModel.extend({ 
  initialize: function() { this.view = null; },
  jsonRoot: "connection",
  isEmailConnection: function() {
    return this.get("connected_user")["email_only"] == true; 
  },

  removeConnection: function(callback) {
    $.post(_u("/api/connections/remove"), {user_id: this.get("connected_user_id")}, callback); 
  }
});

var Connections = RailsCollection.extend({
  url: _u("/api/connections/list.json"),
  jsonRoot: "connection",
  model: Connection,
  
  comparator: function(item) {
    return item.get("connected_user")["name_or_email"].toLowerCase();
  }
});

var PendingConnections = RailsCollection.extend({
  url: _u("/api/connections/waiting.json"),
  jsonRoot: "connection",
  model: Connection
});

var AddRemoveConnections = RailsCollection.extend({
  url: _u("/api/connections/list.json"),
  jsonRoot: "connection",
  model: Connection
});

// Lists
var List = RailsModel.extend({ 
  jsonRoot: "group",
  initialize: function() { 
    this.members = new ListMembers(); //lazy-loaded
    this.members.list = this;

    this.possibleMembers = new PossibleListMembers(); //lazy-loaded
    this.possibleMembers.list = this;
  },

  breadcrumbs: function() {
    total = this.get("members_count");
    people = (total > 1 ? (total+" People") : (total > 0 ? (total+" Person") : "Nobody in List"));
    return ["List", this.get("name"), people];
  },

  removeMember: function(memberId, callback) {
    $.post(_u("/api/groups/remove_user"), 
      {group_id: this.id, id: memberId}, function(data) {
      callback.apply(); 
    });
  },

  addMember: function(memberId, callback) {
    $.post(_u("/api/groups/add_user"), 
      {group_id: this.id, id: memberId}, function(data) {
      callback.apply(); 
    });
  }  
});

var Lists = RailsCollection.extend({
  //url: Receivd.rootUrl+"/api/groups/list.json",
	url: _u("/api/groups/list.json"),
  jsonRoot: "group",
  model: List,
  comparator: function(item) {
    return item.get("name").toLowerCase();
  }
});

var ListMembers = RailsCollection.extend({
  // initialized with List model
  url: function() { return _u("/api/groups/members?id="+this.list.get("id")); },
  jsonRoot: "user",
  model: User,
  comparator: function(user) { return user.get("name_or_email"); }
});

var PossibleListMembers = RailsCollection.extend({
  // initialized with List model
  url: function() { return _u("/api/groups/possible_members?id="+
                           this.list.get("id")); },
  jsonRoot: "user",
  model: User,
  comparator: function(user) { return user.get("name_or_email"); }
});

// Uploaded Files
var UploadedFile = RailsModel.extend({ jsonRoot: "uploaded_file" });

// Inviter
var Inviter = { 
  isEmail: function(query) {
    // contributed by Scott Gonzalez: http://projects.scottsplayground.com/email_address_validation/
    return (/^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i).test(query);
  },

  sendInvite: function(_email, callback) { 
    $.post(_u("/api/connections/add_email.json"), {email: _email}, callback); 
  }
};

var Feedback = {
  post: function(text, callback) {
    $.post(_u("/api/feedback"), {"feedback[description]": text}, function(data) {
      callback.apply(data); 
    });
  }
};
