Array.prototype.remove = function(from, to) {
  var rest = this.slice((to || from) + 1 || this.length);
  this.length = from < 0 ? this.length + from : from;
  return this.push.apply(this, rest);
};

var Notification = Backbone.View.extend({
	render: function() {
		var options = new air.NativeWindowInitOptions();
		options.type = air.NativeWindowType.LIGHTWEIGHT;
		options.systemChrome = air.NativeWindowSystemChrome.NONE;
		
		var visibleBounds = air.Screen.mainScreen.visibleBounds;
		var bounds = new air.Rectangle(left, top, width, height);
		
		var htmlLoader = air.HTMLLoader.createRootWindow( 
				true, //makes the window visible
				options, 
				false // no scrollbars, optional
				//bounds	// optional
			 );
		
		// Make the body tag transparent
		htmlLoader.paintsDefaultBackground = false;
		htmlLoader.addEventListener(air.Event.HTML_DOM_INITIALIZE, function(){
			// Inject the opener, maybe for communication purposes 
			htmlLoader.window.opener = window;
		});
		
		htmlLoader.load(new air.URLRequest('Notification.html'));    
	},
});

var ShareView = Backbone.View.extend({
	initialize: function() {
		var sv = this;
		this.el = $("div#share")[0];
		_.bindAll(this, "render", "openSelectFiles", "selectTheseFiles", 
							"startSending", "setupCombobox", "doneSending", "reset", "removeFile",
							"disableButtons", "enableButtons", "start", "toggleMessage", "toggleAdvanced");
		this.delegateEvents(this.events);

		this.selectedFiles = new Array();
		_.extend(this.selectedFiles, Backbone.Events);
		this.selectedFiles.bind("addFiles", this.selectTheseFiles);

		this.lists = new Lists();
		this.connections = new Connections;

		// async fetch lists, then connections on login, then setup autocomplete
		this.lists.fetch({success: function(d) { 
			//sv.lists.forEach(function(l) { log(l.get("name")); });
			sv.connections.fetch({success: function(d) {
				//sv.connections.forEach(function(c) { log(c.get("connected_user").name_or_email) });
				sv.setupCombobox();
			}});
		}});
	},

	events: {
		"click button.send": "startSending",
		"click button#select_files": "openSelectFiles",
		"click button#reset": "reset",
		"click a#message_or_desc" : "toggleMessage",
    "click a#advanced_options_link": "toggleAdvanced",
		"click a.remove": "removeFile"
	},

	removeFile: function(e) {
		var cid = $(e.target).attr("id");
		log("REMOVING FILE: "+cid);
		var file = _.detect(this.selectedFiles, function(f) { return f.cid == cid });
		this.selectedFiles.remove(this.selectedFiles.indexOf(file));
		$("div#"+cid).remove();
	},

  toggleMessage: function(e) {
    e.preventDefault();
    $("div#message").slideToggle();
  },

  toggleAdvanced: function(e) {
    e.preventDefault();
    $("div#advanced_options").slideToggle();
  },

	doneSending: function() {
		if(this.selectedFiles.length == 1) {
			displayNotification("Done sending the file");
		} else {
			displayNotification("Done sending "+this.selectedFiles.length+" files");
		}

		$(this.el).find("div.step3").html("").append("<button id='reset'>Done Sending Files! Send More Files</button>");
		Receivd.quotaView.render();
	},

	reset: function() {
		this.selectedItem = null;
		this.selectedFiles.length = 0;
		$("#combobox_input").val("");
		$(this.el).find("textarea").val("");
		$(this.el).find("div#selected_files").html(""); 
		$(this.el).find("button#select_files").attr('disabled', '');
		$(this.el).find("div.step3").html("").append("<button class='send'>Send Files</button>");
	},

	openSelectFiles: function() {
		var file = air.File.documentsDirectory;	
		var sv = this;
		file.browseForOpenMultiple("Choose Files to Upload", new runtime.Array());
		file.addEventListener(air.FileListEvent.SELECT_MULTIPLE, function(e) { 
			_.forEach(e.files, function(f) {
				var sf = new SelectedFile({file: f});
				sv.selectedFiles.push(sf); 
			});	
			sv.selectedFiles.trigger("addFiles");
		});
	},

	selectedFileTemplate: "<div id='{{cid}}' class='selected pending'><b>{{fileName}}</b> - {{fileSize}}MB<span class='percent'></span><div class='remove'><a class='remove' id='{{cid}}' href='#'>X</a></div></div>",

	selectTheseFiles: function() {
		var sv = this;
		var totalSize = _.reduce(this.selectedFiles, function(sum, f) { return sum+f.get("file").size }, 0.0);
		log(totalSize);

		var remainingBandwidth = Receivd.quotaModel.remainingBandwidth()*1024.0*1024.0;
		log(remainingBandwidth);

		if(totalSize > remainingBandwidth) {
			alert("You need to upgrade your Kicksend account to send these files. The total size of these files ("+
						$.sprintf("%0.2f", totalSize/1024.0/1024.0)+
						"MB) is greater than the amount of bandwidth you have left for this month ("+
						$.sprintf("%0.2f", Receivd.quotaModel.remainingBandwidth())+"MB)");
			return false;
		};

		log("TOTAL SIZE: "+totalSize);

		_.forEach(this.selectedFiles, function(f) { 
			log("SELECTED: "+f.get("file").nativePath); 
			if($(sv.el).find("div#selected_files div#"+f.cid).length == 0) {
				$(sv.el).find("div#selected_files").append(
					Mustache.to_html(sv.selectedFileTemplate, {
						cid: f.cid, 
						fileSize: $.sprintf("%0.2f", f.get("file").size/1024.0/1024.0), 
						fileName: f.fileName()
					}));
			}
		});	
	},

	disableButtons: function() {
		$(this.el).find("button#select_files").attr('disabled', 'disabled');
		$(this.el).find("button.send").attr('disabled', 'disabled');
	},

	enableButtons: function() {
		$(this.el).find("button#select_files").attr('disabled', '');
		$(this.el).find("button.send").attr('disabled', '');
	},

	setupCombobox: function() {
		$("span#combobox_loading").remove();
		var data = {"results": []};

		this.lists.forEach(function(list) {
			data["results"].push({
				"id": "g-"+list.get('id'), 
				"name": "List > "+list.get("name")});
		});

		this.connections.forEach(function(conn) {
			data["results"].push({
				"id": "c-"+conn.get('id'), 
				"name": conn.get("connected_user").name_or_email});
		});

		var view = this;
		$("#combobox").flexbox(data, {
			watermark: "Enter new e-mail, list or friend",
			showArrow: true, paging: false, width: 396,
			maxVisibleRows: 5,
			noResultsText: '', 
			onSelect: function() {
				view.selectedItem = this.getAttribute("rel"); 
				air.trace(view.selectedItem);
		}});

		$("#combobox_input").blur(function() {
			if(Inviter.isEmail(this.value)) {
				view.selectedEmail = this.value;
			}
		});
	},

	getUploadType: function() {
		air.trace(this.selectedItem);
		return (this.selectedItem.match(/^g-/) ? "g" : "c"); 
	},

	getCurrentId: function() {
		return this.selectedItem.split("-")[1];
	},

	startSending: function() {
		var view = this;
		if(this.selectedFiles.length == 0) {
			return this.openSelectFiles();
		};

		if(this.selectedItem == undefined) {
			if(this.selectedEmail != undefined) {
				var email = this.selectedEmail;
				air.trace("SENDING TO EMAIL: "+email);
				Inviter.sendInvite(email, function(data) {
					var cid = null;
					air.trace(data);
          $.get(_u("/api/connections/for.json"), {user_id: data.user.id}, function(d) {
            cid = d.connection.id;
						air.trace("FETCHED CID: "+cid);
						view.selectedItem = "c-"+cid;

						view.start();
          });
				});	

			} else {
				air.trace("NO EMAIL OR LIST/CONN SELECTED");	
				return false;
			}	
		} else {
			this.start();
		}

	},

	start: function() {
		var sv = this;
		var rootUrl = Receivd.rootUrl;
		this.disableButtons();
		this.selectedFiles.forEach(function(file) {
			var gen_id = Math.floor(Math.random()*10001);
			var old_file = file;
			Receivd.uploadQueue.push(function(afterDoneCallback) {
				log("SENDING: "+file.nativePath());	
				$("div#selected_files div#"+file.cid).addClass("active");

				var type = sv.getUploadType();
				var currentId = sv.getCurrentId();
				log("TYPE: "+type+" - ID: "+currentId);

				var filename = file.nativePath().toString().split(air.File.separator).pop();
	    	var uploadKey = "files/"+type+"/"+currentId+"/"+filename;

				var airOpts = new air.URLVariables();
				$.ajax({type: 'POST', async: false, 
				        url: (_u("/api/groups/upload_auth.json")), 
				        data: {key: uploadKey, from: "air"}, success: function(data){
				  uploadKey = "files/"+type+"/"+currentId+"/"+data.escaped_filename;
					log("S3 KEY: "+uploadKey);

					airOpts.key = uploadKey;
					airOpts.acl = data.acl;
				  airOpts.signature = data.signature;
				  airOpts.policy = data.policy;
				  airOpts.bucket = data.bucket;
				  airOpts["success_action_status"] = '201';
				  airOpts["AwsAccessKeyId"] = data.accesskeyid;
				}});

				if(airOpts.key == null) return false;
				log(airOpts.toString());

				// keeping a reference to the file so it doesn't get swept away by GC :|
				var _file = file;

				var urlRequest = new air.URLRequest(Receivd.rootUrl+"/"+Receivd.bucket+"/");
				urlRequest.method = air.URLRequestMethod.POST;
				urlRequest.data = airOpts; 
				file.file_ref().addEventListener(air.Event.COMPLETE, function(e) {
					log("COMPLETED: "+uploadKey);								

					$("div#selected_files div#"+file.cid+" div.remove").hide();
					$("div#selected_files div#"+file.cid).addClass("completed");
					$("div#selected_files div#"+file.cid).find("span.percent").html(100+"%");

					var filename = uploadKey.split("/").pop();
					var split = filename.split(".");
		  	  var extension = split[split.length-1];
		  	  if(extension) {
		  	    mimeType = mimes[extension.toLowerCase()];
		  	    if(mimeType == undefined) mimeType = "application/octet-stream";
		  	    //console.log("MIME-TYPE: "+mimeType);
		  	  } else {
		  	    mimeType = "application/octet-stream";
		  	  }

					var _data = {
						"file_file_size": file.file_ref().size, 
						"file_file_name": filename, 
						"file_content_type": mimeType 
					};

					if($("textarea#message_or_desc").val() != "" || $("textarea#message_or_desc").val().length > 0) {
						_data.file_description = $("textarea#message_or_desc").val();
					}

					var _url = null;
				  if(type == "g") {
						_url = _u("/api/groups/upload_file.json"); 
						_data.group_id = currentId;
					} else if(type == "c") {
						_url = _u("/api/connections/upload_file.json"); 
						_data.connection_id = currentId;
					}	

					$.ajax({ url: _url, type: "POST", data: _data, 
					  success: function(data) {
							
							xmppStream = data["uploaded_file"]["xmpp_stanza"]; 
	    	    	var xn = $(xmppStream);
	
	    	    	var _text = xn.find("publish:first").html();
	    	    	_text = _text.replace('<?XML:NAMESPACE PREFIX = [default] http://jabber.org/protocol/pubsub NS = "http://jabber.org/protocol/pubsub" />', '');
	
	    	    	stanza = $iq({from: xn.attr("from"), to: xn.attr("to"), id: xn.attr("id"), type: "set"}).
	    	    	    c("pubsub", {xmlns: "http://jabber.org/protocol/pubsub"}).
	    	    	      c("publish", {node: xn.find("publish:first").attr("node")}).
	    	    	        t(_text);
	    	    	connection.send(stanza.tree());

							if(afterDoneCallback) afterDoneCallback(); //call the queue operation
						}
					});

					Receivd.uploadQueue.push(function(afterDoneCallback) {
						sv.doneSending();
						if(afterDoneCallback) afterDoneCallback();
					});
				});

				file.file_ref().addEventListener(air.ProgressEvent.PROGRESS, function(e) {
					var percentCompleted = (e.bytesLoaded/e.bytesTotal)*100;
					log("PROGRESS - "+e.target.nativePath+" - " + percentCompleted);

					$("div#selected_files div#"+file.cid+" div.remove").hide();
					$("div#selected_files div#"+file.cid).
							addClass("active").
							find("span.percent").
							html(Math.floor(percentCompleted)+"%");
				});

				file.file_ref().addEventListener(air.HTTPStatusEvent.HTTP_RESPONSE_STATUS, sv.uploadHttpStatusError); 
				file.file_ref().addEventListener(air.SecurityErrorEvent.SECURITY_ERROR, function(e) {
					$("div#selected_files div#"+file.cid).addClass("failed");
				});

				file.file_ref().addEventListener(air.IOErrorEvent.IO_ERROR, function(e) {
					$("div#selected_files div#"+file.cid).addClass("failed");
				});

				file.file_ref().upload(urlRequest, "file", false);
			});
		});		
	},

	uploadHttpStatusError: function(e) {
		//log("STATUS ERROR "+e.status);
		//_.forEach(e.responseHeaders, function(l) { log("STATUS ERROR: "+l.name+"="+l.value); });
	},

	uploadError: function(e) {
		log("ERROR "+e.text+" "+e.responseText+ " "+e.status);
		log("OTHER: "+e.toString());
		$("div#selected_files div#"+file.cid).addClass("failed");
	}
});

